@@ -141,6 +141,8 @@ class ErrorCodes:
     PREPROCESSFAILURE = 1356
     POSTPROCESSFAILURE = 1357
     MISSINGRELEASEUNPACKED = 1358
+    PANDAQUEUENOTACTIVE = 1359
+    IMAGENOTFOUND = 1360
 
     _error_messages = {
         GENERALERROR: "General pilot error, consult batch log",
@@ -260,7 +262,9 @@ class ErrorCodes:
         COREDUMP: "Core dump detected",
         PREPROCESSFAILURE: "Pre-process command failed",
         POSTPROCESSFAILURE: "Post-process command failed",
-        MISSINGRELEASEUNPACKED: "Missing release setup in unpacked container"
+        MISSINGRELEASEUNPACKED: "Missing release setup in unpacked container",
+        PANDAQUEUENOTACTIVE: "PanDA queue is not active",
+        IMAGENOTFOUND: "Image not found"
     }
 
     put_error_codes = [1135, 1136, 1137, 1141, 1152, 1181]
