@@ -401,8 +401,8 @@ def alrb_wrapper(cmd, workdir, job=None):
 
         # add the jobid to be used as an identifier for the payload running inside the container
         # it is used to identify the pid for the process to be tracked by the memory monitor
-        if 'export PANDAID' not in alrb_setup:
-            alrb_setup += "export PANDAID=%s;" % job.jobid
+        if 'export PandaID' not in alrb_setup:
+            alrb_setup += "export PandaID=%s;" % job.jobid
         log.debug('alrb_setup=%s' % alrb_setup)
 
         # add TMPDIR
@@ -439,6 +439,11 @@ def alrb_wrapper(cmd, workdir, job=None):
 
         # add atlasLocalSetup command + options (overwrite the old cmd since the new cmd is the containerised version)
         cmd = add_asetup(job, alrb_setup, queuedata.is_cvmfs, release_setup, container_script, queuedata.container_options, new_mode)
+
+        # add any container options if set
+        execargs = job.containeroptions.get('execArgs', None)
+        if execargs:
+            cmd += ' ' + execargs
         log.debug('\n\nfinal command:\n\n%s\n' % cmd)
     else:
         log.warning('container name not defined in AGIS')
@@ -699,7 +704,7 @@ def singularity_wrapper(cmd, workdir, job=None):
         queuedata = infoservice.queuedata
 
     container_name = queuedata.container_type.get("pilot")  # resolve container name for user=pilot
-    logger.debug("resolved container_name from queuedata.contaner_type: %s" % container_name)
+    logger.debug("resolved container_name from queuedata.container_type: %s" % container_name)
 
     if container_name == 'singularity':
         logger.info("singularity has been requested")
@@ -737,23 +742,24 @@ def singularity_wrapper(cmd, workdir, job=None):
     return cmd
 
 
-def create_stagein_container_command(workdir, cmd):
+def create_middleware_container_command(workdir, cmd, label='stagein'):
     """
-    Create the stage-in container command.
+    Create the stage-in/out container command.
 
-    The function takes the isolated stage-in command, adds bits and pieces needed for the containerisation and stores
-    it in a stagein.sh script file. It then generates the actual command that will execute the stage-in script in a
+    The function takes the isolated stage-in/out command, adds bits and pieces needed for the containerisation and stores
+    it in a stage[in|out].sh script file. It then generates the actual command that will execute the stage-in/out script in a
     container.
 
     new cmd:
       lsetup rucio davis xrootd
       old cmd
       exit $?
-    write new cmd to stagein.sh script
+    write new cmd to stage[in|out].sh script
     create container command and return it
 
     :param workdir: working directory where script will be stored (string).
-    :param cmd: isolated stage-in command (string).
+    :param cmd: isolated stage-in/out command (string).
+    :param label: 'stage-[in|out]' (string).
     :return: container command to be executed (string).
     """
 
@@ -764,7 +770,7 @@ def create_stagein_container_command(workdir, cmd):
     logger.debug('setup.sh content:\n%s' % content)
 
     # store it in setup.sh
-    script_name = 'stagein.sh'
+    script_name = 'stagein.sh' if label == 'stage-in' else 'stageout.sh'
     try:
         status = write_file(os.path.join(workdir, script_name), content)
     except PilotException as e:
